import test from 'tape'
import xhtm from 'xhtm'

import vhtml from './index.js'
const html = xhtm.bind(vhtml)

test('vhtml', t => {
  t.equal(
    html`
      <script>
        alert("hello" + ', world!')
      </script>
    `,
    `<script>alert("hello" + ', world!')</script>`,
    'should not escape contents of script tags'
  )

  t.equal(
    html`
      <style>
        p::before {
          content: "before"
        }
        p::after {
          content: 'after'
        }
      </style>
    `,
    `<style>p::before {content: "before"}p::after {content: 'after'}</style>`,
    'should not escape contents of style tags'
  )

  t.equal(
    html`<small>&copy; 2023-present Aral Balkan</small>`,
    '<small>&copy; 2023-present Aral Balkan</small>',
    'should leave HTML entities alone'
  )

  // Note: This fails currently because xhtm throws a syntax error on the
  // solitary less than character in the content.
  // See: https://github.com/dy/xhtm/issues/20

  // t.equal(
  //   html`<p>This < that > other thing</p>`,
  //   '<p>This &lt; that > other thing</p>',
  //   'should escape solitary less than characters in text content'
  // )

  // Should stringify HTML.

  let items = ['one', 'two', 'three']
  
  t.equal(
    html`
      <div class="foo">
        <h1>Hi!</h1>
        <p>Here is a list of ${items.length} items:</p>
        <ul>
          ${
            items.map( item => (
              html`<li>${ item }</li>`
            ))
          }
        </ul>
      </div>
    `,
    '<div class="foo"><h1>Hi!</h1><p>Here is a list of 3 items:</p><ul><li>one</li><li>two</li><li>three</li></ul></div>',
    'should stringify html'
  )

  t.equal(html`<div><div onclick=${`&<>"'`}></div></div>`, '<div><div onclick="&amp;&lt;&gt;&quot;&apos;"></div></div>', 'should sanitise attributes')

  t.equal(
    html`<style>p::before { content: 'something'; }</style>`,
    `<style>p::before { content: 'something'; }</style>`,
    'should not sanitise <style> tags'
  )

  t.equal(
    html`<p hidden>I am hidden.</p>`,
    '<p hidden>I am hidden.</p>',
    'should correctly emit boolean HTML attributes'
  )

  t.equal(
    html`
      <div>
        ${[['a','b']]}
        <c>d</c>
        ${['e',['f'],[['g']]]}
      </div>
    `,
    '<div>ab<c>d</c>efg</div>',
    'should flatten children'
  )

  items = ['one', 'two']

  let Item = ({ item, index, children }) => html`
    <li id=${index}>
      <h4>${item}</h4>
      ${children}
    </li>
  `
  
  t.equal(
    html`
      <div class="foo">
        <h1>Hi!</h1>
        <ul>
          ${
            items.map((item, index) => html`
              <${Item} item=${item} index=${index}>
                This is item ${item}!
              </${Item}>
            `)
          }
        </ul>
      </div>
    `,
    '<div class="foo"><h1>Hi!</h1><ul><li id="0"><h4>one</h4>This is item one!</li><li id="1"><h4>two</h4>This is item two!</li></ul></div>',
    'should support sort-of components'
  )

  Item = () => html`
    <li>
      <h4></h4>
    </li>
  `

  t.equal(
    html`
      <div class="foo">
        <h1>Hi!</h1>
        <ul>
          ${
            items.map(item => html`
              <${Item}>
                This is item ${item}!
              </${Item}>
            `)
          }
        </ul>
      </div>
    `,
    '<div class="foo"><h1>Hi!</h1><ul><li><h4></h4></li><li><h4></h4></li></ul></div>',
    'should support sort-of components without arguments'
  )  

  Item = ({ children }) => html`
    <li>
      <h4></h4>
      ${children}
    </li>
  `

  t.equal(
    html`
      <div class="foo">
        <h1>Hi!</h1>
        <ul>
          ${
            items.map(item => html`
              <${Item}>
                This is item ${item}!
              </${Item}>
            `)
          }
        </ul>
      </div>
    `,
    '<div class="foo"><h1>Hi!</h1><ul><li><h4></h4>This is item one!</li><li><h4></h4>This is item two!</li></ul></div>',
    'should support sort-of components without args but with children'
  )  

  t.equal(
    html`
      <div>
        <area />
        <base />
        <br />
        <col />
        <command />
        <embed />
        <hr />
        <img />
        <input />
        <keygen />
        <link />
        <meta />
        <param />
        <source />
        <track />
        <wbr />
        <!-- Not void elements -->
        <div />
        <span />
        <p />
      </div>
    `,
    '<div><area><base><br><col><command><embed><hr><img><input><keygen><link><meta><param><source><track><wbr><div></div><span></span><p></p></div>',
    'should support empty (void) tags'
  )

  t.equal(html`<div className="my-class" htmlFor="id" />`, '<div class="my-class" for="id"></div>', 'should handle special prop names')

  t.equal(vhtml(null, null, "foo", "bar", "baz"), 'foobarbaz', 'should support string fragments')

  t.equal(vhtml(null, null, html`<p>foo</p>`, html`<em>bar</em>`, html`<div class="qqqqqq">baz</div>`), '<p>foo</p><em>bar</em><div class="qqqqqq">baz</div>', 'should support element fragments')

  t.end()
})
